@@ -57,7 +57,6 @@ class Input_file_lib;
 class Position_dependent_options;
 class Target;
 class Plugin_manager;
-class Script_info;
 
 // Incremental build action for a specific file, as selected by the user.
 
@@ -624,10 +623,6 @@ class General_options
   DEFINE_bool_alias(Bstatic, Bdynamic, options::ONE_DASH, '\0',
 		    N_("-l does not search for shared libraries"), NULL,
 		    true);
-  DEFINE_bool_alias(dy, Bdynamic, options::ONE_DASH, '\0',
-		    N_("alias for -Bdynamic"), NULL, false);
-  DEFINE_bool_alias(dn, Bdynamic, options::ONE_DASH, '\0',
-		    N_("alias for -Bstatic"), NULL, true);
 
   DEFINE_bool(Bsymbolic, options::ONE_DASH, '\0', false,
               N_("Bind defined symbols locally"), NULL);
@@ -761,6 +756,9 @@ class General_options
   DEFINE_string(soname, options::ONE_DASH, 'h', NULL,
                 N_("Set shared library name"), N_("FILENAME"));
 
+  DEFINE_bool(i, options::EXACTLY_ONE_DASH, '\0', false,
+	      N_("Ignored"), NULL);
+
   DEFINE_double(hash_bucket_empty_fraction, options::TWO_DASHES, '\0', 0.0,
 		N_("Min fraction of empty buckets in dynamic hash"),
 		N_("FRACTION"));
@@ -787,11 +785,6 @@ class General_options
   DEFINE_special(incremental_update, options::TWO_DASHES, '\0',
 		 N_("Do an incremental link; exit if not possible"), NULL);
 
-  DEFINE_string(incremental_base, options::TWO_DASHES, '\0', NULL,
-                N_("Set base file for incremental linking"
-                   " (default is output file)"),
-                N_("FILE"));
-
   DEFINE_special(incremental_changed, options::TWO_DASHES, '\0',
                  N_("Assume files changed"), NULL);
 
@@ -898,8 +891,6 @@ class General_options
 
   DEFINE_bool(relocatable, options::EXACTLY_ONE_DASH, 'r', false,
               N_("Generate relocatable output"), NULL);
-  DEFINE_bool_alias(i, relocatable, options::EXACTLY_ONE_DASH, '\0',
-		    N_("Synonym for -r"), NULL, false);
 
   DEFINE_bool(relax, options::TWO_DASHES, '\0', false,
 	      N_("Relax branches on certain targets"), NULL);
@@ -1524,7 +1515,7 @@ class Input_file_argument
   //         command line, such as --whole-archive.
   Input_file_argument()
     : name_(), type_(INPUT_FILE_TYPE_FILE), extra_search_path_(""),
-      just_symbols_(false), options_(), arg_serial_(0)
+      just_symbols_(false), options_()
   { }
 
   Input_file_argument(const char* name, Input_file_type type,
@@ -1532,7 +1523,7 @@ class Input_file_argument
                       bool just_symbols,
                       const Position_dependent_options& options)
     : name_(name), type_(type), extra_search_path_(extra_search_path),
-      just_symbols_(just_symbols), options_(options), arg_serial_(0)
+      just_symbols_(just_symbols), options_(options)
   { }
 
   // You can also pass in a General_options instance instead of a
@@ -1544,7 +1535,7 @@ class Input_file_argument
                       bool just_symbols,
                       const General_options& options)
     : name_(name), type_(type), extra_search_path_(extra_search_path),
-      just_symbols_(just_symbols), options_(options), arg_serial_(0)
+      just_symbols_(just_symbols), options_(options)
   { }
 
   const char*
@@ -1586,16 +1577,6 @@ class Input_file_argument
 	    || !this->extra_search_path_.empty());
   }
 
-  // Set the serial number for this argument.
-  void
-  set_arg_serial(unsigned int arg_serial)
-  { this->arg_serial_ = arg_serial; }
-
-  // Get the serial number.
-  unsigned int
-  arg_serial() const
-  { return this->arg_serial_; }
-
  private:
   // We use std::string, not const char*, here for convenience when
   // using script files, so that we do not have to preserve the string
@@ -1605,8 +1586,6 @@ class Input_file_argument
   std::string extra_search_path_;
   bool just_symbols_;
   Position_dependent_options options_;
-  // A unique index for this file argument in the argument list.
-  unsigned int arg_serial_;
 };
 
 // A file or library, or a group, from the command line.
@@ -1616,17 +1595,17 @@ class Input_argument
  public:
   // Create a file or library argument.
   explicit Input_argument(Input_file_argument file)
-    : is_file_(true), file_(file), group_(NULL), lib_(NULL), script_info_(NULL)
+    : is_file_(true), file_(file), group_(NULL), lib_(NULL)
   { }
 
   // Create a group argument.
   explicit Input_argument(Input_file_group* group)
-    : is_file_(false), group_(group), lib_(NULL), script_info_(NULL)
+    : is_file_(false), group_(group), lib_(NULL)
   { }
 
   // Create a lib argument.
   explicit Input_argument(Input_file_lib* lib)
-    : is_file_(false), group_(NULL), lib_(lib), script_info_(NULL)
+    : is_file_(false), group_(NULL), lib_(lib)
   { }
 
   // Return whether this is a file.
@@ -1684,22 +1663,11 @@ class Input_argument
     return this->lib_;
   }
 
-  // If a script generated this argument, store a pointer to the script info.
-  // Currently used only for recording incremental link information.
-  void
-  set_script_info(Script_info* info)
-  { this->script_info_ = info; }
-
-  Script_info*
-  script_info() const
-  { return this->script_info_; }
-
  private:
   bool is_file_;
   Input_file_argument file_;
   Input_file_group* group_;
   Input_file_lib* lib_;
-  Script_info* script_info_;
 };
 
 typedef std::vector<Input_argument> Input_argument_list;
@@ -1717,12 +1685,9 @@ class Input_file_group
   { }
 
   // Add a file to the end of the group.
-  Input_argument&
+  void
   add_file(const Input_file_argument& arg)
-  {
-    this->files_.push_back(Input_argument(arg));
-    return this->files_.back();
-  }
+  { this->files_.push_back(Input_argument(arg)); }
 
   // Iterators to iterate over the group contents.
 
@@ -1751,12 +1716,9 @@ class Input_file_lib
   { }
 
   // Add a file to the end of the lib.
-  Input_argument&
+  void
   add_file(const Input_file_argument& arg)
-  {
-    this->files_.push_back(Input_argument(arg));
-    return this->files_.back();
-  }
+  { this->files_.push_back(Input_argument(arg)); }
 
   const Position_dependent_options&
   options() const
@@ -1789,12 +1751,12 @@ class Input_arguments
   typedef Input_argument_list::const_iterator const_iterator;
 
   Input_arguments()
-    : input_argument_list_(), in_group_(false), in_lib_(false), file_count_(0)
+    : input_argument_list_(), in_group_(false), in_lib_(false)
   { }
 
   // Add a file.
-  Input_argument&
-  add_file(Input_file_argument& arg);
+  void
+  add_file(const Input_file_argument& arg);
 
   // Start a group (the --start-group option).
   void
@@ -1842,18 +1804,10 @@ class Input_arguments
   empty() const
   { return this->input_argument_list_.empty(); }
 
-  // Return the number of input files.  This may be larger than
-  // input_argument_list_.size(), because of files that are part
-  // of groups or libs.
-  int
-  number_of_input_files() const
-  { return this->file_count_; }
-
  private:
   Input_argument_list input_argument_list_;
   bool in_group_;
   bool in_lib_;
-  unsigned int file_count_;
 };
 
 
@@ -1910,7 +1864,7 @@ class Command_line
   // The number of input files.
   int
   number_of_input_files() const
-  { return this->inputs_.number_of_input_files(); }
+  { return this->inputs_.size(); }
 
   // Iterators to iterate over the list of input files.
 
